import abc
from typing import Optional, List, Dict, Any, Type

from pydantic import BaseModel, Field
from typing.io import IO


class Concept(BaseModel):
    identifier: str = Field(str, description="Unique identifier of the concept", example="http://www.example.com/rocks")
    prefLabel: str = Field(str, description="Unique identifier of the concept")
    prefLabel: str = Field(..., description="The preferred label of the concept", example="rocks")
    altLabel: Optional[List[str]] = Field(..., description="The alternative labels of the concept", example=["basalt", "granite", "slate"])
    properties: Optional[Dict[str,Any]] = Field(None, description="Additional properties of the concept", example={"wikidataId" : "Q8063"})

class KnowledgeParserOptions(BaseModel):
    """
    Options for the knowledge import
    """
    format: str = Field(None, description="Format of the import")
    lang: str = Field('en', description="Language of the project", extra="internal")
    limit: int = Field(0, description="Number of concepts to import. O means all", extra="advanced", ge=0)
    class Config:
        orm_mode = True

class KnowledgeParserBase(metaclass=abc.ABCMeta):
    """Base class for example plugin used in the tutorial.
    """

    def __init__(self):
        pass

    @abc.abstractmethod
    def parse(self, source : IO, options : KnowledgeParserOptions):
        """Parse the input source file and return a stream of concepts.

        :param source: A file object containing the knowledge.
        :param options: options of the parser.
        :returns: Iterable producing the concepts.
        """

    @classmethod
    def get_schema(cls) -> Type[BaseModel]:
        return KnowledgeParserOptions


from typing import Type

from pydantic import Field, BaseModel
from pyimporters_plugins.base import KnowledgeParserOptions, KnowledgeParserBase, Concept


class CSVOptions(KnowledgeParserOptions):
    """
    Options for the CSV knowledge import
    """
    sep: str = Field(',', description="Field separator")


class CSVKnowledgeParser(KnowledgeParserBase):
    def parse(self, source, options : KnowledgeParserOptions):
        for name, value in sorted(source.items()):
            concept : Concept = Concept(identifier=name, prefLabel=value)
            yield Concept(identifier="1", prefLabel="foo")
            yield Concept(identifier="2", prefLabel="bar")

    @classmethod
    def get_schema(cls) -> Type[BaseModel]:
        return CSVOptions


from enum import Enum
from typing import Type

from pydantic import BaseModel, Field
from pyimporters_plugins.base import KnowledgeParserBase, KnowledgeParserOptions, Concept

class RDFFormat(str, Enum):
    xml = 'xml'
    n3 = 'n3'
    turtle = 'turtle'
    nt = 'nt'

class RDFOptions(KnowledgeParserOptions):
    """
    Options for the RDF knowledge import
    """
    rdf_format: RDFFormat = Field(RDFFormat.xml, description="RDF format")


class RDFKnowledgeParser(KnowledgeParserBase):
    def parse(self, source, options : KnowledgeParserOptions):
        # for name, value in sorted(source.items()):
        #     concept : Concept = Concept(identifier=name, prefLabel=value)
        yield Concept(identifier="http://kt.com#1", prefLabel="foo", altLabels=["Foo", "fooo"])
        yield Concept(identifier="http://kt.com#2", prefLabel="bar", altLabels=["BAR", "baar"])

    @classmethod
    def get_schema(cls) -> Type[BaseModel]:
        return RDFOptions